@@ -41,6 +41,7 @@ struct v_or<T1, T2, Tail...>
     , v_or<T2, Tail...>
     >::type
 {
+    static_assert(sizeof...(Tail) != 0, "Impossible!");
 };
 
 /**
@@ -58,7 +59,7 @@ struct v_or<T> : boost::mpl::bool_<T::type::value>
 namespace sample { namespace details {
 constexpr std::size_t pow_bytes(const std::size_t what, const unsigned d)
 {
-    return d ? 1024 * pow_bytes(what, d - 1) : what;
+    return d ? 1'024 * pow_bytes(what, d - 1) : what;
 }
 }                                                           // namespace details
 
@@ -85,6 +86,7 @@ constexpr std::size_t operator"" GiB(const unsigned long long size)
 
 constexpr std::size_t BUFFER_SIZE = 10_Mib;                 // user defined literal (const)
 constexpr std::size_t MESSAGE_SIZE = 100_Kib;
+constexpr std::size_t MAX_PKT_LENGTH = 100'500_bytes;       // user defined literal w/ delimiters
 extern thread_local std::string s_thread_name;              // prefixed static variable
 extern std::string g_app_name;                              // prefixed global variable
 // names w/ leading underscore are reserved by C++ Standard
@@ -130,14 +132,22 @@ const std::string h = u8"привет"_RU;                        // user define
 
 namespace numbers {
 constexpr int a = 123;                                      // decimal
+constexpr int a1 = -123'456;                                // decimal w/ delimiters
 constexpr int au = 123u;                                    // unsigned decimal
 constexpr long al = 123l;                                   // long decimal
 constexpr long al = 123l;                                   // long decimal
 constexpr long long all = 123ll;                            // long long decimal
-constexpr unsigned long long aull = 123ull;                 // unsigned long long decimal
+constexpr unsigned long long aull = +123ull;                // unsigned long long decimal
 constexpr auto a_invalid = 123uull;
 constexpr int b = 0123;                                     // octal
-constexpr int c = 0x123;                                    // hex
+constexpr int b1 = -0'123'456;                              // octal w/ delimiters
+constexpr int octal_invalid = -0678;
+constexpr auto c = 0x123;                                   // hex
+constexpr auto c = 0x1234'5678'9abc;                        // hex w/ delimiters
+constexpr auto z = 0b1010110001110;                         // binary w/ delimiters
+constexpr auto z1 = 0b1'0101'1000'1110;                     // binary w/ delimiters
+constexpr auto binary_invalid = 0b012;
+
 
 const auto d = 10.;
 const auto e = 10.01;
@@ -177,7 +187,12 @@ class derived : public base
     {
         std::cout << __PRETTY_FUNCTION__ << ": " << __DATE__<< std::endl;
     }
-    void exit(int a) [[noreturn]]                           // C++11 attributes
+    // C++11 attributes
+    void exit() [[noreturn, deprecated("Use exit(int) instead")]]
+    {
+        exit(other_);
+    }
+    void exit(int a) [[noreturn, gcc::visibility("default")]]
     {
         /// GCC builtins
         if (__builtin_expect(a == 0, 1))
@@ -185,8 +200,8 @@ class derived : public base
             // ...
         }
     }
+    alignas(long) int other_;                               // google code style compatible member name
     int m_member;                                           // prefixed data member
-    int other_;                                             // google code style compatible member name
 };
 
 // Commented by preprocessor
@@ -205,6 +220,6 @@ std::string comment = "comment";
 char other_commented;
 #endif
 
-// Modelines: switch to C++11 mode
-// kate: hl C++11;
+// Modelines: switch to C++ mode
+// kate: hl C++;
 // kate: indent-width 4;
