@@ -1157,6 +1157,7 @@ KateViewConfig::KateViewConfig ()
    m_persistentSelectionSet (false),
    m_viInputModeSet (false),
    m_viInputModeStealKeysSet (false),
+   m_viRelativeLineNumbersSet (false),
    m_viInputModeEmulateCommandBarSet(false),
    m_automaticCompletionInvocationSet (false),
    m_wordCompletionSet (false),
@@ -1166,6 +1167,7 @@ KateViewConfig::KateViewConfig ()
    m_scrollPastEndSet (false),
    m_allowMarkMenu (true),
    m_wordCompletionRemoveTailSet (false),
+   m_foldFirstLineSet (false),
    m_view (0)
 {
   s_global = this;
@@ -1198,6 +1200,7 @@ KateViewConfig::KateViewConfig (KateView *view)
    m_persistentSelectionSet (false),
    m_viInputModeSet (false),
    m_viInputModeStealKeysSet (false),
+   m_viRelativeLineNumbersSet (false),
    m_viInputModeEmulateCommandBarSet(false),
    m_automaticCompletionInvocationSet (false),
    m_wordCompletionSet (false),
@@ -1207,6 +1210,7 @@ KateViewConfig::KateViewConfig (KateView *view)
    m_scrollPastEndSet (false),
    m_allowMarkMenu (true),
    m_wordCompletionRemoveTailSet (false),
+   m_foldFirstLineSet (false),
    m_view (view)
 {
 }
@@ -1240,6 +1244,7 @@ namespace
   const char * const KEY_PERSISTENT_SELECTION = "Persistent Selection";
   const char * const KEY_VI_INPUT_MODE = "Vi Input Mode";
   const char * const KEY_VI_INPUT_MODE_STEAL_KEYS = "Vi Input Mode Steal Keys";
+  const char * const KEY_VI_RELATIVE_LINE_NUMBERS = "Vi Relative Line Numbers";
   const char * const KEY_VI_INPUT_MODE_EMULATE_COMMAND_BAR = "Vi Input Mode Emulate Command Bar";
   const char * const KEY_AUTOMATIC_COMPLETION_INVOCATION = "Auto Completion";
   const char * const KEY_WORD_COMPLETION = "Word Completion";
@@ -1248,6 +1253,7 @@ namespace
   const char * const KEY_WORD_COMPLETION_REMOVE_TAIL = "Word Completion Remove Tail";
   const char * const KEY_SMART_COPY_CUT = "Smart Copy Cut";
   const char * const KEY_SCROLL_PAST_END = "Scroll Past End";
+  const char * const KEY_FOLD_FIRST_LINE = "Fold First Line";
 }
 
 void KateViewConfig::readConfig ( const KConfigGroup &config)
@@ -1293,6 +1299,7 @@ void KateViewConfig::readConfig ( const KConfigGroup &config)
 
   setViInputMode (config.readEntry( KEY_VI_INPUT_MODE, false));
   setViInputModeStealKeys (config.readEntry( KEY_VI_INPUT_MODE_STEAL_KEYS, false));
+  setViRelativeLineNumbers(config.readEntry( KEY_VI_RELATIVE_LINE_NUMBERS, false));
   setViInputModeEmulateCommandBar (config.readEntry( KEY_VI_INPUT_MODE_EMULATE_COMMAND_BAR, false));
 
   setAutomaticCompletionInvocation (config.readEntry( KEY_AUTOMATIC_COMPLETION_INVOCATION, true ));
@@ -1302,6 +1309,7 @@ void KateViewConfig::readConfig ( const KConfigGroup &config)
   setWordCompletionRemoveTail (config.readEntry( KEY_WORD_COMPLETION_REMOVE_TAIL, true ));
   setSmartCopyCut (config.readEntry( KEY_SMART_COPY_CUT, false ));
   setScrollPastEnd (config.readEntry( KEY_SCROLL_PAST_END, false ));
+  setFoldFirstLine (config.readEntry( KEY_FOLD_FIRST_LINE, false ));
 
   if (isGlobal()) {
     // Read search pattern history
@@ -1362,11 +1370,11 @@ void KateViewConfig::writeConfig (KConfigGroup &config)
 
   config.writeEntry( KEY_SMART_COPY_CUT, smartCopyCut() );
   config.writeEntry( KEY_SCROLL_PAST_END , scrollPastEnd() );
+  config.writeEntry( KEY_FOLD_FIRST_LINE, foldFirstLine() );
 
   config.writeEntry( KEY_VI_INPUT_MODE, viInputMode());
-
   config.writeEntry( KEY_VI_INPUT_MODE_STEAL_KEYS, viInputModeStealKeys());
-
+  config.writeEntry( KEY_VI_RELATIVE_LINE_NUMBERS, viRelativeLineNumbers());
   config.writeEntry( KEY_VI_INPUT_MODE_EMULATE_COMMAND_BAR, viInputModeEmulateCommandBar());
 
 
@@ -1832,6 +1840,24 @@ void KateViewConfig::setViInputModeStealKeys (bool on)
   configEnd ();
 }
 
+bool KateViewConfig::viRelativeLineNumbers() const
+{
+  if (m_viRelativeLineNumbersSet  || isGlobal())
+    return m_viRelativeLineNumbers;
+
+  return s_global->viRelativeLineNumbers();
+}
+
+void KateViewConfig::setViRelativeLineNumbers(bool on)
+{
+  if (m_viRelativeLineNumbersSet && m_viRelativeLineNumbers ==  on)
+    return;
+
+  configStart();
+  m_viRelativeLineNumbersSet = true;
+  m_viRelativeLineNumbers = on;
+  configEnd();
+}
 
 bool KateViewConfig::viInputModeEmulateCommandBar() const
 {
@@ -1989,6 +2015,27 @@ void KateViewConfig::setScrollPastEnd (bool on)
   configEnd ();
 }
 
+bool KateViewConfig::foldFirstLine() const
+{
+  if (m_foldFirstLineSet || isGlobal())
+    return m_foldFirstLine;
+
+  return s_global->foldFirstLine();
+}
+
+void KateViewConfig::setFoldFirstLine(bool on)
+{
+  if (m_foldFirstLineSet && m_foldFirstLine == on)
+    return;
+
+  configStart ();
+
+  m_foldFirstLineSet = true;
+  m_foldFirstLine = on;
+
+  configEnd ();
+}
+
 //END
 
 //BEGIN KateRendererConfig
