@@ -505,6 +505,9 @@ class KATEPART_TESTS_EXPORT KateViewConfig : public KateConfig
     bool viInputModeStealKeys () const;
     void setViInputModeStealKeys (bool on);
 
+    bool viRelativeLineNumbers() const;
+    void setViRelativeLineNumbers(bool on);
+
     bool viInputModeEmulateCommandBar() const;
     void setViInputModeEmulateCommandBar(bool on);
 
@@ -539,6 +542,9 @@ class KATEPART_TESTS_EXPORT KateViewConfig : public KateConfig
     bool scrollPastEnd() const;
     void setScrollPastEnd(bool on);
 
+    bool foldFirstLine() const;
+    void setFoldFirstLine(bool on);
+
   private:
     bool m_dynWordWrap;
     int m_dynWordWrapIndicators;
@@ -562,6 +568,7 @@ class KATEPART_TESTS_EXPORT KateViewConfig : public KateConfig
     bool m_persistentSelection;
     bool m_viInputMode;
     bool m_viInputModeStealKeys;
+    bool m_viRelativeLineNumbers;
     bool m_viInputModeEmulateCommandBar;
     bool m_automaticCompletionInvocation;
     bool m_wordCompletion;
@@ -570,6 +577,7 @@ class KATEPART_TESTS_EXPORT KateViewConfig : public KateConfig
     bool m_wordCompletionRemoveTail;
     bool m_smartCopyCut;
     bool m_scrollPastEnd;
+    bool m_foldFirstLine;
 
     bool m_dynWordWrapSet : 1;
     bool m_dynWordWrapIndicatorsSet : 1;
@@ -590,6 +598,7 @@ class KATEPART_TESTS_EXPORT KateViewConfig : public KateConfig
     bool m_persistentSelectionSet : 1;
     bool m_viInputModeSet : 1;
     bool m_viInputModeStealKeysSet : 1;
+    bool m_viRelativeLineNumbersSet : 1;
     bool m_viInputModeEmulateCommandBarSet : 1;
     bool m_automaticCompletionInvocationSet : 1;
     bool m_wordCompletionSet : 1;
@@ -599,6 +608,7 @@ class KATEPART_TESTS_EXPORT KateViewConfig : public KateConfig
     bool m_scrollPastEndSet : 1;
     bool m_allowMarkMenu : 1;
     bool m_wordCompletionRemoveTailSet : 1;
+    bool m_foldFirstLineSet : 1;
 
   private:
     static KateViewConfig *s_global;
